@@ -1,12 +1,8 @@
 from neurograd import xp
-from base import Function
 from typing import TYPE_CHECKING, Union, Tuple, Sequence, Literal
 from numpy.typing import ArrayLike
 if TYPE_CHECKING:
     from neurograd.tensor import Tensor
-from neurograd.functions.tensor_ops import Pad, SlidingWindowView, Reshape
-from neurograd.functions.linalg import MatMul, Transpose
-
 
 
 
@@ -86,22 +82,13 @@ def conv2d(input: Union["Tensor", xp.ndarray], filters: Union["Tensor", xp.ndarr
             axes=(0, 3, 1, 2)
         )
     else:
-        # Depthwise convolution - each channel gets its own filter
+        # Depthwise convolution
         # slides: (N, C, out_H, out_W, F_H, F_W)
         # filters: (C, F_H, F_W)
-        # Reshape slides -> (N*out_H*out_W, C, F_H*F_W)
-        slides_reshaped = ng.reshape(
-            slides,
-            (N * out_H * out_W, C, F_H * F_W)
-        )
-        # Reshape filters -> (C, F_H*F_W, 1)
-        filters_reshaped = ng.reshape(filters, (C, F_H * F_W, 1))
-        # Batched matmul: (N*out_H*out_W, C, F_H*F_W) @ (C, F_H*F_W, 1)
-        # Result: (N*out_H*out_W, C, 1)
-        output = ng.matmul(slides_reshaped, filters_reshaped)
-        # Reshape back -> (N, C, out_H, out_W)
-        output = ng.reshape(output, (N, out_H, out_W, C))
-        output = ng.transpose(output, (0, 3, 1, 2))  # (N, C, out_H, out_W)
+        output = ng.sum(
+            slides * filters.reshape(1, C, 1, 1, F_H, F_W),
+            axis=(4, 5)
+        )  # (N, C, out_H, out_W)
     return output
 
 
