@@ -2,37 +2,23 @@ from neurograd import xp
 import builtins
 from .base import Function
 from neurograd.nn.module import Module
-from functools import reduce
-import operator
+
 
 def _reduce(arr, axis, reduction_func, keepdims=False, **kwargs):
-    """
-    Helper to perform reductions over multiple axes. Uses a
-    transpose-and-reshape strategy to convert multi-axis reduction
-    into an efficient single-axis reduction, avoiding CuPy slowdown
-    and iterative reduction overhead.
-    """
-    if axis is None:
-        # Reduce over all axes
-        result = reduction_func(arr.ravel(), axis=0, **kwargs)
-        return result.reshape((1,) * arr.ndim) if keepdims else result
-    if isinstance(axis, int):
-        # Single-axis reduction
+    """Helper to perform reductions over multiple axes iteratively for CuPy compatibility."""
+    if axis is None or isinstance(axis, int):
         return reduction_func(arr, axis=axis, keepdims=keepdims, **kwargs)
-    # --- Multi-axis reduction ("Reshape and Conquer") ---
-    reduce_axes = tuple(sorted(ax if ax >= 0 else arr.ndim + ax for ax in axis))
-    keep_axes = tuple(i for i in range(arr.ndim) if i not in reduce_axes)
-    # Transpose and reshape
-    permuted = arr.transpose(keep_axes + reduce_axes)
-    reshaped = permuted.reshape(-1, reduce(operator.mul, (arr.shape[i] for i in reduce_axes), 1))
-    # Reduce over combined axis
-    result = reduction_func(reshaped, axis=1, **kwargs)
-    # Reshape back
-    if keepdims:
-        shape = list(arr.shape)
-        for ax in reduce_axes: shape[ax] = 1
-        return result.reshape(shape)
-    return result.reshape([arr.shape[i] for i in keep_axes] if keep_axes else ())
+    # For tuple of axes, reduce iteratively
+    ndim = arr.ndim
+    axes = tuple(ax if ax >= 0 else ndim + ax for ax in axis)
+    axes = tuple(sorted(axes, reverse=True))
+    result = arr
+    for ax in axes:
+        result = reduction_func(result, axis=ax, keepdims=True, **kwargs)
+    if not keepdims:
+        result = xp.squeeze(result, axis=axes)
+    return result
+
 
 class Sum(Function, Module):
     name = "Sum"
@@ -76,17 +62,17 @@ class Mean(Function, Module):
         if self.axis is None:
             n = x.size
         else:
-            if isinstance(self.axis, int):
-                n = x.shape[self.axis]
-            else:
-                n = 1
-                for ax in self.axis:
-                    n *= x.shape[ax]
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            n = 1
+            for ax in axes:
+                ax_norm = ax if ax >= 0 else len(x.shape) + ax
+                n *= x.shape[ax_norm]
         # In-place division to avoid intermediate array
-        grad_output /= n
+        grad_output = grad_output.copy() / n
         # Expand and broadcast gradient
         if self.axis is not None and not self.keepdims:
-            for ax in sorted(self.axis if isinstance(self.axis, tuple) else (self.axis,)):
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            for ax in sorted(axes):
                 ax_norm = ax if ax >= 0 else len(x.shape) + ax
                 grad_output = xp.expand_dims(grad_output, axis=ax_norm)
         return xp.broadcast_to(grad_output, x.shape)
@@ -104,7 +90,12 @@ class Max(Function, Module):
         if self.keepdims:
             return self.max_vals
         else:
-            return xp.squeeze(self.max_vals, axis=self.axis)
+            # Convert axis to positive indices for squeezing
+            if self.axis is None:
+                return self.max_vals.reshape(())
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            axes = tuple(ax if ax >= 0 else x.ndim + ax for ax in axes)
+            return xp.squeeze(self.max_vals, axis=axes)
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
         if not x.requires_grad:
@@ -113,7 +104,7 @@ class Max(Function, Module):
         # Use self.max_vals which already has keepdims=True for broadcasting
         mask = (x.data == self.max_vals).astype(x.data.dtype)
         if self.axis is None:
-            mask /= _reduce(mask, None, xp.sum)  # In-place division
+            mask /= xp.sum(mask)  # In-place division
         else:
             count = _reduce(mask, self.axis, xp.sum, keepdims=True)
             count = xp.where(count == 0, 1, count)
@@ -121,16 +112,12 @@ class Max(Function, Module):
         # Expand and broadcast gradient
         grad = grad_output
         if self.axis is not None and not self.keepdims:
-            ndim = x.ndim
-            if isinstance(self.axis, int):
-                axis_list = [self.axis if self.axis >= 0 else ndim + self.axis]
-            else:
-                axis_list = [ax if ax >= 0 else ndim + ax for ax in self.axis]
-            axis_list = sorted(axis_list)
-            for ax in axis_list:
-                grad = xp.expand_dims(grad, axis=ax)
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            for ax in sorted(axes):
+                ax_norm = ax if ax >= 0 else len(x.shape) + ax
+                grad = xp.expand_dims(grad, axis=ax_norm)
         grad = xp.broadcast_to(grad, x.shape)
-        grad *= mask  # In-place multiplication
+        grad = grad * mask  # Use explicit multiplication to avoid in-place modification
         return grad
 
 class Min(Function, Module):
@@ -146,7 +133,12 @@ class Min(Function, Module):
         if self.keepdims:
             return self.min_vals
         else:
-            return xp.squeeze(self.min_vals, axis=self.axis)
+            # Convert axis to positive indices for squeezing
+            if self.axis is None:
+                return self.min_vals.reshape(())
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            axes = tuple(ax if ax >= 0 else x.ndim + ax for ax in axes)
+            return xp.squeeze(self.min_vals, axis=axes)
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
         if not x.requires_grad:
@@ -155,7 +147,7 @@ class Min(Function, Module):
         # Use self.min_vals which already has keepdims=True for broadcasting  
         mask = (x.data == self.min_vals).astype(x.data.dtype)
         if self.axis is None:
-            mask /= _reduce(mask, None, xp.sum)  # In-place division
+            mask /= xp.sum(mask)  # In-place division
         else:
             count = _reduce(mask, self.axis, xp.sum, keepdims=True)
             count = xp.where(count == 0, 1, count)
@@ -163,16 +155,12 @@ class Min(Function, Module):
         # Expand and broadcast gradient
         grad = grad_output
         if self.axis is not None and not self.keepdims:
-            ndim = x.ndim
-            if isinstance(self.axis, int):
-                axis_list = [self.axis if self.axis >= 0 else ndim + self.axis]
-            else:
-                axis_list = [ax if ax >= 0 else ndim + ax for ax in self.axis]
-            axis_list = sorted(axis_list)
-            for ax in axis_list:
-                grad = xp.expand_dims(grad, axis=ax)
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            for ax in sorted(axes):
+                ax_norm = ax if ax >= 0 else len(x.shape) + ax
+                grad = xp.expand_dims(grad, axis=ax_norm)
         grad = xp.broadcast_to(grad, x.shape)
-        grad *= mask  # In-place multiplication
+        grad = grad * mask  # Use explicit multiplication to avoid in-place modification
         return grad
 
 class Std(Function, Module):
@@ -185,11 +173,18 @@ class Std(Function, Module):
         self.eps = eps
     def forward(self, x: xp.ndarray) -> xp.ndarray:
         self.mean = _reduce(x, self.axis, xp.mean, keepdims=True, dtype=xp.float32)
-        self.std_vals = _reduce(x, self.axis, xp.std, keepdims=True, dtype=xp.float32)
+        centered = x - self.mean
+        self.var = _reduce(centered**2, self.axis, xp.mean, keepdims=True, dtype=xp.float32)
+        self.std_vals = xp.sqrt(self.var + self.eps)
         if self.keepdims:
             return self.std_vals
         else:
-            return xp.squeeze(self.std_vals, axis=self.axis)
+            # Convert axis to positive indices for squeezing
+            if self.axis is None:
+                return self.std_vals.reshape(())
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            axes = tuple(ax if ax >= 0 else x.ndim + ax for ax in axes)
+            return xp.squeeze(self.std_vals, axis=axes)
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
         if not x.requires_grad:
@@ -201,24 +196,24 @@ class Std(Function, Module):
         if self.axis is None:
             n = x.size
         else:
-            axes = (self.axis,) if isinstance(self.axis, int) else tuple(self.axis)
-            axes = tuple(a if a >= 0 else x.ndim + a for a in axes)
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
             n = 1
-            for a in axes:
-                n *= x.shape[a]
+            for ax in axes:
+                ax_norm = ax if ax >= 0 else x.ndim + ax
+                n *= x.shape[ax_norm]
         # Avoid divide-by-zero
         std_safe = xp.where(std == 0, self.eps, std)
         base_grad = (x.data - mean) / (n * std_safe)
         # If keepdims=False, expand grad_output back along reduced axes
         go = grad_output
         if self.axis is not None and not self.keepdims:
-            axes = (self.axis,) if isinstance(self.axis, int) else tuple(self.axis)
-            axes = tuple(a if a >= 0 else x.ndim + a for a in axes)
-            for a in sorted(axes):
-                go = xp.expand_dims(go, axis=a)
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            for ax in sorted(axes):
+                ax_norm = ax if ax >= 0 else x.ndim + ax
+                go = xp.expand_dims(go, axis=ax_norm)
         # Broadcast and apply chain rule
         go = xp.broadcast_to(go, x.shape)
-        base_grad *= go  # In-place multiplication
+        base_grad = base_grad * go  # Use explicit multiplication to avoid in-place modification
         return base_grad
 
 class Var(Function, Module):
@@ -232,7 +227,19 @@ class Var(Function, Module):
         self.eps = eps
     def forward(self, x: xp.ndarray) -> xp.ndarray:
         self.mean = _reduce(x, self.axis, xp.mean, keepdims=True, dtype=xp.float32)
-        var_vals = _reduce(x, self.axis, xp.var, keepdims=self.keepdims, ddof=self.ddof, dtype=xp.float32)
+        centered = x - self.mean
+        var_vals = _reduce(centered**2, self.axis, xp.mean, keepdims=self.keepdims, dtype=xp.float32)
+        # Apply Bessel's correction if needed
+        if self.ddof > 0:
+            if self.axis is None:
+                n = x.size
+            else:
+                axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+                n = 1
+                for ax in axes:
+                    ax_norm = ax if ax >= 0 else x.ndim + ax
+                    n *= x.shape[ax_norm]
+            var_vals = var_vals * n / (n - self.ddof)
         return var_vals
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
@@ -243,13 +250,12 @@ class Var(Function, Module):
         # Count elements along the reduced axes
         if self.axis is None:
             n = x.size
-            axes = tuple(range(x.ndim))
         else:
-            axes = (self.axis,) if isinstance(self.axis, int) else tuple(self.axis)
-            axes = tuple(a if a >= 0 else x.ndim + a for a in axes)
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
             n = 1
-            for a in axes:
-                n *= x.shape[a]
+            for ax in axes:
+                ax_norm = ax if ax >= 0 else x.ndim + ax
+                n *= x.shape[ax_norm]
         # Denominator n - ddof, avoid divide-by-zero or negative
         denom = builtins.max(n - self.ddof, self.eps)
         # d/dx var(x) = 2(x - mean)/(n - ddof)
@@ -258,11 +264,13 @@ class Var(Function, Module):
         # If keepdims=False, expand grad_output back along reduced axes
         go = grad_output
         if self.axis is not None and not self.keepdims:
-            for a in sorted(axes):
-                go = xp.expand_dims(go, axis=a)
+            axes = (self.axis,) if isinstance(self.axis, int) else self.axis
+            for ax in sorted(axes):
+                ax_norm = ax if ax >= 0 else x.ndim + ax
+                go = xp.expand_dims(go, axis=ax_norm)
         # Broadcast and apply chain rule
         go = xp.broadcast_to(go, x.shape)
-        base_grad *= go  # In-place multiplication
+        base_grad = base_grad * go  # Use explicit multiplication to avoid in-place modification
         return base_grad
 
 def sum(x, axis=None, keepdims=False):
@@ -276,4 +284,4 @@ def min(x, axis=None, keepdims=False):
 def std(x, axis=None, keepdims=False):
     return Std(axis=axis, keepdims=keepdims)(x)
 def var(x, axis=None, keepdims=False):
-    return Var(axis=axis, keepdims=keepdims)(x)
+    return Var(axis=axis, keepdims=keepdims)(x)
\ No newline at end of file
